@@ -11,6 +11,9 @@ calls it with the rendered model.
 :param js_modules: URLs of JS modules making up Bokeh library
 :type js_modules: list
 
+:param js_exports: URLs of JS modules to be exported
+:type js_exports: dict
+
 :param css_urls: CSS urls to inject
 :type css_urls: list
 
@@ -21,13 +24,13 @@ calls it with the rendered model.
   }
 
   var force = {{ force|default(False)|json }};
+  var py_version = '{{ version }}'.replace('rc', '-rc.');
+  var is_dev = py_version.indexOf("+") !== -1 || py_version.indexOf("-") !== -1;
+  var reloading = {{ reloading|default(False)|json }};
+  var Bokeh = root.Bokeh;
+  var bokeh_loaded = Bokeh != null && (Bokeh.version === py_version || (Bokeh.versions !== undefined && Bokeh.versions.has(py_version)));
 
-  if (typeof root._bokeh_onload_callbacks === "undefined" || force === true) {
-    root._bokeh_onload_callbacks = [];
-    root._bokeh_is_loading = undefined;
-  }
-
-  if (typeof (root._bokeh_timeout) === "undefined" || force === true) {
+  if (typeof (root._bokeh_timeout) === "undefined" || force) {
     root._bokeh_timeout = Date.now() + {{ timeout|default(0)|json }};
     root._bokeh_failed_load = false;
   }
@@ -39,26 +42,30 @@ calls it with the rendered model.
           callback();
       });
     } finally {
-      delete root._bokeh_onload_callbacks
+      delete root._bokeh_onload_callbacks;
     }
     console.debug("Bokeh: all callbacks have finished");
   }
 
-  function load_libs(css_urls, js_urls, js_modules, callback) {
+  function load_libs(css_urls, js_urls, js_modules, js_exports, callback) {
     if (css_urls == null) css_urls = [];
     if (js_urls == null) js_urls = [];
     if (js_modules == null) js_modules = [];
+    if (js_exports == null) js_exports = {};
 
     root._bokeh_onload_callbacks.push(callback);
+
     if (root._bokeh_is_loading > 0) {
       console.debug("Bokeh: BokehJS is being loaded, scheduling callback at", now());
       return null;
     }
-    if (js_urls.length === 0 && js_modules.length === 0) {
+    if (js_urls.length === 0 && js_modules.length === 0 && Object.keys(js_exports).length === 0) {
       run_callbacks();
       return null;
     }
-    console.debug("Bokeh: BokehJS not loaded, scheduling load and callback at", now());
+    if (!reloading) {
+      console.debug("Bokeh: BokehJS not loaded, scheduling load and callback at", now());
+    }
 
     function on_load() {
       root._bokeh_is_loading--;
@@ -67,23 +74,12 @@ calls it with the rendered model.
         run_callbacks()
       }
     }
+    window._bokeh_on_load = on_load
 
     function on_error() {
       console.error("failed to load " + url);
     }
 
-    for (var i = 0; i < css_urls.length; i++) {
-      var url = css_urls[i];
-      const element = document.createElement("link");
-      element.onload = on_load;
-      element.onerror = on_error;
-      element.rel = "stylesheet";
-      element.type = "text/css";
-      element.href = url;
-      console.debug("Bokeh: injecting link tag for BokehJS stylesheet: ", url);
-      document.body.appendChild(element);
-    }
-
     var skip = [];
     if (window.requirejs) {
       window.requirejs.config({{ config|conffilter }});
@@ -97,8 +93,33 @@ calls it with the rendered model.
       {% endfor %}
       root._bokeh_is_loading = css_urls.length + {{ requirements|length }};
     } else {
-      root._bokeh_is_loading = css_urls.length + js_urls.length + js_modules.length;
+      root._bokeh_is_loading = css_urls.length + js_urls.length + js_modules.length + Object.keys(js_exports).length;
     }
+
+    var existing_stylesheets = []
+    var links = document.getElementsByTagName('link')
+    for (var i = 0; i < links.length; i++) {
+      var link = links[i]
+      if (link.href != null) {
+	existing_stylesheets.push(link.href)
+      }
+    }
+    for (var i = 0; i < css_urls.length; i++) {
+      var url = css_urls[i];
+      if (existing_stylesheets.indexOf(url) !== -1) {
+	on_load()
+	continue;
+      }
+      const element = document.createElement("link");
+      element.onload = on_load;
+      element.onerror = on_error;
+      element.rel = "stylesheet";
+      element.type = "text/css";
+      element.href = url;
+      console.debug("Bokeh: injecting link tag for BokehJS stylesheet: ", url);
+      document.body.appendChild(element);
+    }
+
     {%- for lib, urls in skip_imports.items() %}
     if (((window['{{ lib }}'] !== undefined) && (!(window['{{ lib }}'] instanceof HTMLElement))) || window.requirejs) {
       var urls = {{ urls }};
@@ -107,9 +128,17 @@ calls it with the rendered model.
       }
     }
     {%- endfor %}
+    var existing_scripts = []
+    var scripts = document.getElementsByTagName('script')
+    for (var i = 0; i < scripts.length; i++) {
+      var script = scripts[i]
+      if (script.src != null) {
+	existing_scripts.push(script.src)
+      }
+    }
     for (var i = 0; i < js_urls.length; i++) {
       var url = js_urls[i];
-      if (skip.indexOf(url) >= 0) {
+      if (skip.indexOf(url) !== -1 || existing_scripts.indexOf(url) !== -1) {
 	if (!window.requirejs) {
 	  on_load();
 	}
@@ -125,7 +154,7 @@ calls it with the rendered model.
     }
     for (var i = 0; i < js_modules.length; i++) {
       var url = js_modules[i];
-      if (skip.indexOf(url) >= 0) {
+      if (skip.indexOf(url) !== -1 || existing_scripts.indexOf(url) !== -1) {
 	if (!window.requirejs) {
 	  on_load();
 	}
@@ -140,6 +169,26 @@ calls it with the rendered model.
       console.debug("Bokeh: injecting script tag for BokehJS library: ", url);
       document.head.appendChild(element);
     }
+    for (const name in js_exports) {
+      var url = js_exports[name];
+      if (skip.indexOf(url) >= 0 || root[name] != null) {
+	if (!window.requirejs) {
+	  on_load();
+	}
+	continue;
+      }
+      var element = document.createElement('script');
+      element.onerror = on_error;
+      element.async = false;
+      element.type = "module";
+      console.debug("Bokeh: injecting script tag for BokehJS library: ", url);
+      element.textContent = `
+      import ${name} from "${url}"
+      window.${name} = ${name}
+      window._bokeh_on_load()
+      `
+      document.head.appendChild(element);
+    }
     if (!js_urls.length && !js_modules.length) {
       on_load()
     }
@@ -153,6 +202,7 @@ calls it with the rendered model.
 
   var js_urls = {{ bundle.js_urls|json }};
   var js_modules = {{ bundle.js_modules|json }};
+  var js_exports = {{ bundle.js_module_exports|json }};
   var css_urls = {{ bundle.css_urls|json }};
   var inline_js = [
     {%- for css in bundle.css_raw %}
@@ -160,7 +210,7 @@ calls it with the rendered model.
       inject_raw_css({{ css|json }});
     },
     {%- endfor %}
-    {%- for js in bundle.js_raw %}
+    {%- for js in (bundle.js_raw if bundle else js_raw) %}
     function(Bokeh) {
       {{ js|indent(6) }}
     },
@@ -173,6 +223,17 @@ calls it with the rendered model.
       for (var i = 0; i < inline_js.length; i++) {
         inline_js[i].call(root, root.Bokeh);
       }
+      // Cache old bokeh versions
+      if (Bokeh != undefined && !reloading) {
+	var NewBokeh = root.Bokeh;
+	if (Bokeh.versions === undefined) {
+	  Bokeh.versions = new Map();
+	}
+	if (NewBokeh.version !== Bokeh.version) {
+	  Bokeh.versions.set(NewBokeh.version, NewBokeh)
+	}
+	root.Bokeh = Bokeh;
+      }
       {%- if elementid -%}
       if (force === true) {
         display_loaded();
@@ -184,15 +245,39 @@ calls it with the rendered model.
       console.log("Bokeh: BokehJS failed to load within specified timeout.");
       root._bokeh_failed_load = true;
     }
+    root._bokeh_is_initializing = false
   }
 
-  if (root._bokeh_is_loading === 0) {
-    console.debug("Bokeh: BokehJS loaded, going straight to plotting");
-    run_inline_js();
-  } else {
-    load_libs(css_urls, js_urls, js_modules, function() {
-      console.debug("Bokeh: BokehJS plotting callback run at", now());
-      run_inline_js();
-    });
+  function load_or_wait() {
+    // Implement a backoff loop that tries to ensure we do not load multiple
+    // versions of Bokeh and its dependencies at the same time.
+    // In recent versions we use the root._bokeh_is_initializing flag
+    // to determine whether there is an ongoing attempt to initialize
+    // bokeh, however for backward compatibility we also try to ensure
+    // that we do not start loading a newer (Panel>=1.0 and Bokeh>3) version
+    // before older versions are fully initialized.
+    if (root._bokeh_is_initializing && Date.now() > root._bokeh_timeout) {
+      root._bokeh_is_initializing = false;
+      root._bokeh_onload_callbacks = undefined;
+      console.log("Bokeh: BokehJS was loaded multiple times but one version failed to initialize.");
+      load_or_wait();
+    } else if (root._bokeh_is_initializing || (typeof root._bokeh_is_initializing === "undefined" && root._bokeh_onload_callbacks !== undefined)) {
+      setTimeout(load_or_wait, 100);
+    } else {
+      Bokeh = root.Bokeh;
+      bokeh_loaded = Bokeh != null && (Bokeh.version === py_version || (Bokeh.versions !== undefined && Bokeh.versions.has(py_version)));
+      root._bokeh_is_initializing = true
+      root._bokeh_onload_callbacks = []
+      if (!reloading && (!bokeh_loaded || is_dev)) {
+	root.Bokeh = undefined;
+      }
+      load_libs(css_urls, js_urls, js_modules, js_exports, function() {
+	console.debug("Bokeh: BokehJS plotting callback run at", now());
+	run_inline_js();
+      });
+    }
   }
+  // Give older versions of the autoload script a head-start to ensure
+  // they initialize before we start loading newer version.
+  setTimeout(load_or_wait, 100)
 }(window));
