@@ -25,7 +25,7 @@ from bokeh.server.contexts import ApplicationContext
 from tornado.ioloop import PeriodicCallback
 from tornado.web import StaticFileHandler
 
-from ..auth import OAuthProvider
+from ..auth import BasicProvider, OAuthProvider
 from ..config import config
 from ..io.document import _cleanup_doc
 from ..io.liveness import LivenessHandler
@@ -92,6 +92,11 @@ class Serve(_BkServe):
             help=("Static directories to serve specified as key=value "
                   "pairs mapping from URL route to static file directory.")
         )),
+        ('--basic-auth', dict(
+            action = 'store',
+            type   = str,
+            help   = "Password or filepath to use with Basic Authentication."
+        )),
         ('--oauth-provider', dict(
             action = 'store',
             type   = str,
@@ -143,6 +148,11 @@ class Serve(_BkServe):
             type    = str,
             help    = "Template to serve when user is unauthenticated."
         )),
+        ('--basic-login-template', dict(
+            action  = 'store',
+            type    = str,
+            help    = "Template to serve for Basic Authentication login page."
+        )),
         ('--rest-provider', dict(
             action = 'store',
             type   = str,
@@ -172,6 +182,12 @@ class Serve(_BkServe):
             action  = 'store_true',
             help    = "Whether to add an admin panel."
         )),
+        ('--admin-log-level', dict(
+            action  = 'store',
+            default = None,
+            choices = ('debug', 'info', 'warning', 'error', 'critical'),
+            help    = "One of: debug (default), info, warning, error or critical",
+        )),
         ('--profiler', dict(
             action  = 'store',
             type    = str,
@@ -203,10 +219,18 @@ class Serve(_BkServe):
             help    = "The endpoint for the liveness API.",
             default = "liveness"
         )),
+        ('--reuse-sessions', dict(
+            action  = 'store_true',
+            help    = "Whether to reuse sessions when serving the initial request.",
+        )),
+        ('--global-loading-spinner', dict(
+            action  = 'store_true',
+            help    = "Whether to add a global loading spinner to the application(s).",
+        )),
     )
 
     # Supported file extensions
-    _extensions = ['.py']
+    _extensions = ['.py', '.ipynb', '.md']
 
     def customize_applications(self, args, applications):
         if args.index and not args.index.endswith('.html'):
@@ -218,6 +242,20 @@ class Serve(_BkServe):
                 applications['/'] = applications[f'/{index}']
         return super().customize_applications(args, applications)
 
+    def warm_applications(self, applications, reuse_sessions):
+        from ..io.session import generate_session
+        for path, app in applications.items():
+            session = generate_session(app)
+            with set_curdoc(session.document):
+                if config.session_key_func:
+                    reuse_sessions = False
+                else:
+                    state._session_key_funcs[path] = lambda r: r.path
+                    state._sessions[path] = session
+                    session.block_expiration()
+                state._on_load(None)
+            _cleanup_doc(session.document, destroy=not reuse_sessions)
+
     def customize_kwargs(self, args, server_kwargs):
         '''Allows subclasses to customize ``server_kwargs``.
 
@@ -226,6 +264,8 @@ class Serve(_BkServe):
         kwargs = dict(server_kwargs)
         if 'index' not in kwargs:
             kwargs['index'] = INDEX_HTML
+        elif kwargs['index'].endswith('.html'):
+            kwargs['index'] = os.path.abspath(kwargs['index'])
 
         # Handle tranquilized functions in the supplied functions
         kwargs['extra_patterns'] = patterns = kwargs.get('extra_patterns', [])
@@ -269,6 +309,8 @@ class Serve(_BkServe):
             raise ValueError("rest-provider %r not recognized." % args.rest_provider)
 
         config.autoreload = args.autoreload
+        config.global_loading_spinner = args.global_loading_spinner
+        config.reuse_sessions = args.reuse_sessions
 
         if config.autoreload:
             for f in files:
@@ -291,17 +333,11 @@ class Serve(_BkServe):
             applications = build_single_handler_applications(files, argvs)
             if args.autoreload:
                 with record_modules():
-                    for app in applications.values():
-                        doc = app.create_document()
-                        with set_curdoc(doc):
-                            state._on_load(None)
-                        _cleanup_doc(doc)
+                    self.warm_applications(
+                        applications, args.reuse_sessions
+                    )
             else:
-                for app in applications.values():
-                    doc = app.create_document()
-                    with set_curdoc(doc):
-                        state._on_load(None)
-                    _cleanup_doc(doc)
+                self.warm_applications(applications, args.reuse_sessions)
 
         if args.liveness:
             argvs = {f: args.args for f in files}
@@ -343,6 +379,15 @@ class Serve(_BkServe):
             except Exception:
                 pass
             patterns.extend(app_patterns)
+            if args.admin_log_level is not None:
+                if os.environ.get('PANEL_ADMIN_LOG_LEVEL'):
+                    raise ValueError(
+                        "admin_log_level supplied both using the environment variable "
+                        "PANEL_ADMIN_LOG_LEVEL and as an explicit argument, only the "
+                        "value supplied to the environment variable is used "
+                    )
+                else:
+                    config.admin_log_level = args.admin_log_level.upper()
 
         config.session_history = args.session_history
         if args.rest_session_info:
@@ -360,12 +405,42 @@ class Serve(_BkServe):
 
         if args.auth_template:
             authpath = pathlib.Path(args.auth_template)
-            if not authpath.isfile():
+            if not authpath.is_file():
                 raise ValueError(
-                    "The supplied auth-template {args.auth_template} does not "
+                    f"The supplied auth-template {args.auth_template} does not "
                     "exist, ensure you supply and existing Jinja2 template."
                 )
             config.auth_template = str(authpath.absolute())
+
+        if args.basic_auth and config.basic_auth:
+            raise ValueError(
+                "Turn on Basic authentication using environment variable "
+                "or via explicit argument, not both"
+            )
+        if args.basic_auth:
+            config.basic_auth = args.basic_auth
+        if config.basic_auth:
+            if args.basic_login_template:
+                basic_login_template = args.basic_login_template
+                authpath = pathlib.Path(basic_login_template)
+                if not authpath.is_file():
+                    raise ValueError(
+                        f"The supplied auth-template {basic_login_template} does not "
+                        "exist, ensure you supply and existing Jinja2 template."
+                    )
+            else:
+                basic_login_template = None
+            kwargs['auth_provider'] = BasicProvider(basic_login_template=basic_login_template)
+
+        if args.cookie_secret and config.cookie_secret:
+            raise ValueError(
+                "Supply cookie secret either using environment "
+                "variable or via explicit argument, not both."
+            )
+        elif args.cookie_secret:
+            config.cookie_secret = args.cookie_secret
+
+        # Check only one auth is used.
         if args.oauth_provider and config.oauth_provider:
                 raise ValueError(
                     "Supply OAuth provider either using environment variable "
@@ -390,6 +465,14 @@ class Serve(_BkServe):
                     "variable."
                 )
 
+            if not config.cookie_secret:
+                raise ValueError(
+                    "When enabling an OAuth provider you must supply "
+                    "a valid cookie_secret either using the --cookie-secret "
+                    "CLI argument or the PANEL_COOKIE_SECRET environment "
+                    "variable."
+                )
+
             if config.oauth_secret and args.oauth_secret:
                 raise ValueError(
                     "Supply OAuth secret either using environment variable "
@@ -450,20 +533,6 @@ class Serve(_BkServe):
                     )
                 state.encryption = Fernet(config.oauth_encryption_key)
 
-            if args.cookie_secret and config.cookie_secret:
-                raise ValueError(
-                    "Supply cookie secret either using environment "
-                    "variable or via explicit argument, not both."
-                )
-            elif args.cookie_secret:
-                config.cookie_secret = args.cookie_secret
-            elif not config.cookie_secret:
-                raise ValueError(
-                    "When enabling an OAuth provider you must supply "
-                    "a valid cookie_secret either using the --cookie-secret "
-                    "CLI argument or the PANEL_COOKIE_SECRET environment "
-                    "variable."
-                )
             if args.oauth_error_template:
                 error_template = str(pathlib.Path(args.oauth_error_template).absolute())
             elif config.auth_template:
