@@ -1,7 +1,10 @@
+import sys
+
 import param
 
 from param.parameterized import iscoroutinefunction
 
+from .util import eval_function
 from .widgets import Widget
 
 ipywidget_classes = {}
@@ -10,6 +13,12 @@ ipywidget_classes = {}
 def param_value_if_widget(arg):
     if isinstance(arg, Widget):
         return arg.param.value
+    elif 'hvplot.interactive' in sys.modules:
+        from hvplot.interactive import Interactive
+        if isinstance(arg, Interactive):
+            def interactive_eval(*args, **kwargs):
+                return arg.eval()
+            return bind(interactive_eval, *arg._params)
 
     from .pane.ipywidget import IPyWidget
     if IPyWidget.applies(arg) and hasattr(arg, 'value'):
@@ -30,13 +39,14 @@ def depends(*args, **kwargs):
     Python decorator annotating a function or `Parameterized` method to
     express its dependencies on a set of Parameters.
 
+    Despite still being available usage of `pn.depends` is no longer
+    recommended, in favor of the less intrusive `pn.bind`.
+
     Returns a "reactive" function that binds (some of) its arguments to
     Parameter values. This means that the "reactive" function can
     (or will if `watch=True`) be automatically invoked whenever the underlying
     parameter values change.
 
-    See also `pn.bind`.
-
     Reference: https://panel.holoviz.org/user_guide/APIs.html#reactive-functions
 
     :Example:
@@ -156,8 +166,9 @@ def _param_bind(function, *args, watch=False, **kwargs):
     annotated with all dependencies.
     """
     dependencies = {}
-    for i, arg in enumerate(args):
-        p = param_value_if_widget(arg)
+    if isinstance(function, (param.Parameter, Widget)):
+        dependencies['__fn'] = param_value_if_widget(function)
+    for i, p in enumerate(args):
         if hasattr(p, '_dinfo'):
             for j, arg in enumerate(p._dinfo['dependencies']):
                 dependencies[f'__arg{i}_arg{j}'] = arg
@@ -166,20 +177,19 @@ def _param_bind(function, *args, watch=False, **kwargs):
         elif isinstance(p, param.Parameter):
             dependencies[f'__arg{i}'] = p
     for kw, v in kwargs.items():
-        p = param_value_if_widget(v)
-        if hasattr(p, '_dinfo'):
-            for j, arg in enumerate(p._dinfo['dependencies']):
+        if hasattr(v, '_dinfo'):
+            for j, arg in enumerate(v._dinfo['dependencies']):
                 dependencies[f'__kwarg_{kw}_arg{j}'] = arg
-            for pkw, kwarg in p._dinfo['kw'].items():
+            for pkw, kwarg in v._dinfo['kw'].items():
                 dependencies[f'__kwarg_{kw}_{pkw}'] = kwarg
-        elif isinstance(p, param.Parameter):
-            dependencies[kw] = p
+        elif isinstance(v, param.Parameter):
+            dependencies[kw] = v
 
     def combine_arguments(wargs, wkwargs):
         combined_args = []
         for arg in args:
             if hasattr(arg, '_dinfo'):
-                arg = arg()
+                arg = eval_function(arg)
             elif isinstance(arg, param.Parameter):
                 arg = getattr(arg.owner, arg.name)
             combined_args.append(arg)
@@ -187,26 +197,38 @@ def _param_bind(function, *args, watch=False, **kwargs):
         combined_kwargs = {}
         for kw, arg in kwargs.items():
             if hasattr(arg, '_dinfo'):
-                arg = arg()
+                arg = eval_function(arg)
             elif isinstance(arg, param.Parameter):
                 arg = getattr(arg.owner, arg.name)
             combined_kwargs[kw] = arg
         for kw, arg in wkwargs.items():
-            if kw.startswith('__arg') or kw.startswith('__kwarg'):
+            if kw.startswith('__arg') or kw.startswith('__kwarg') or kw.startswith('__fn'):
                 continue
             combined_kwargs[kw] = arg
         return combined_args, combined_kwargs
 
+    def eval_fn():
+        if callable(function):
+            fn = function
+        else:
+            p = param_value_if_widget(function)
+            if isinstance(p, param.Parameter):
+                fn = getattr(p.owner, p.name)
+            else:
+                fn = eval_function(p)
+        return fn
+
     if iscoroutinefunction(function):
         @depends(**dependencies, watch=watch)
         async def wrapped(*wargs, **wkwargs):
             combined_args, combined_kwargs = combine_arguments(wargs, wkwargs)
-            return await function(*combined_args, **combined_kwargs)
+
+            return await eval_fn()(*combined_args, **combined_kwargs)
     else:
         @depends(**dependencies, watch=watch)
         def wrapped(*wargs, **wkwargs):
             combined_args, combined_kwargs = combine_arguments(wargs, wkwargs)
-            return function(*combined_args, **combined_kwargs)
+            return eval_fn()(*combined_args, **combined_kwargs)
     wrapped.__bound_function__ = function
     return wrapped
 
