@@ -109,9 +109,11 @@ def get_server_root_dir(settings):
 EXECUTION_TEMPLATE = """
 import os
 import pathlib
+import sys
 
-app = '{{ path }}'
+app = r'{{ path }}'
 os.chdir(str(pathlib.Path(app).parent))
+sys.path = [os.getcwd()] + sys.path[1:]
 
 from panel.io.jupyter_executor import PanelExecutor
 executor = PanelExecutor(app, '{{ token }}', '{{ root_url }}')
@@ -183,7 +185,9 @@ class PanelJupyterHandler(JupyterHandler):
             elif msg_type == 'comm_open' and msg['content']['target_name'] == self.session_id:
                 comm_id = msg['content']['comm_id']
             elif msg_type == 'stream' and msg['content']['name'] == 'stderr':
-                logger.info(msg['content']['text'])
+                logger.error(msg['content']['text'])
+            elif msg_type == "error":
+                logger.error(msg['content']['traceback'])
         return result, comm_id, extension_dirs
 
     @tornado.web.authenticated
@@ -233,6 +237,7 @@ class PanelJupyterHandler(JupyterHandler):
                 )
                 self.finish(html)
                 return
+
         kernel_env = {**os.environ}
         kernel_id = await ensure_async(
             (
@@ -380,10 +385,12 @@ class PanelWSProxy(WSHandler, JupyterHandler):
                 if not await ensure_async(self.kernel.is_alive()):
                     raise RuntimeError("Kernel died before expected shutdown of Panel app.")
                 continue
-            if not (
-                msg['header']['msg_type'] == 'comm_msg' and
-                msg['content']['comm_id'] == self.comm_id
-            ):
+
+            msg_type = msg['header']['msg_type']
+            if msg_type == 'stream' and msg['content']['name'] == 'stderr':
+                logger.error(msg['content']['text'])
+                continue
+            elif not (msg_type == 'comm_msg' and msg['content']['comm_id'] == self.comm_id):
                 continue
             content, metadata = msg['content'], msg['metadata']
             status = metadata.get('status')
