@@ -45,9 +45,10 @@ def _stdlibs():
 _STDLIBS = _stdlibs()
 _PACKAGE_MAP = {
     'sklearn': 'scikit-learn',
-    'hvplot': ['holoviews>=1.15.4', 'hvplot'],
-    'holoviews': ['holoviews>=1.15.4']
+    'hvplot': ['holoviews>=>=1.16.0a7', 'hvplot'],
+    'holoviews': ['holoviews>=1.16.0a7']
 }
+_IGNORED_PKGS = ['js', 'pyodide']
 _PANDAS_AUTODETECT = ['bokeh.sampledata', 'as_frame']
 
 def find_imports(code: str) -> List[str]:
@@ -97,7 +98,7 @@ def find_imports(code: str) -> List[str]:
             packages.append(pkg)
     if any(pdd in code for pdd in _PANDAS_AUTODETECT) and 'pandas' not in packages:
         packages.append('pandas')
-    return packages
+    return [pkg for pkg in packages if pkg not in _IGNORED_PKGS]
 
 #---------------------------------------------------------------------
 # Execution API
@@ -284,6 +285,8 @@ def format_mime(obj):
     for method, mime_type in reversed(list(MIME_METHODS.items())):
         if mime_type in format_dict:
             output = format_dict[mime_type]
+        elif isinstance(obj, type) and method != '__repr__':
+            output = None
         else:
             output = eval_formatter(obj, method)
 
