@@ -5,17 +5,18 @@ inside the Jupyter notebook.
 from __future__ import annotations
 
 import json
-import uuid
 import sys
+import uuid
 
-from contextlib import contextmanager
 from collections import OrderedDict
+from contextlib import contextmanager
 from typing import (
-    TYPE_CHECKING, Any, Dict, List, Iterator, Optional, Tuple
+    TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Tuple,
 )
 
 import bokeh
 import bokeh.embed.notebook
+import param
 
 from bokeh.core.json_encoder import serialize_json
 from bokeh.core.templates import MACROS
@@ -24,19 +25,21 @@ from bokeh.embed import server_document
 from bokeh.embed.elements import div_for_render_item, script_for_render_items
 from bokeh.embed.util import standalone_docs_json_and_render_items
 from bokeh.embed.wrappers import wrap_in_script_tag
-from bokeh.models import LayoutDOM, Model
+from bokeh.models import Model
 from bokeh.resources import CDN, INLINE
-from bokeh.settings import settings, _Unset
+from bokeh.settings import _Unset, settings
 from bokeh.util.serialization import make_id
 from pyviz_comms import (
-    PYVIZ_PROXY, Comm, JupyterCommManager as _JupyterCommManager, nb_mime_js
+    PYVIZ_PROXY, Comm, JupyterCommJS,
+    JupyterCommManager as _JupyterCommManager, nb_mime_js,
 )
 
-from ..compiler import require_components
 from ..util import escape
 from .embed import embed_state
 from .model import add_to_doc, diff
-from .resources import PANEL_DIR, Bundle, Resources, _env, bundle_resources
+from .resources import (
+    PANEL_DIR, Resources, _env, bundle_resources, patch_model_css,
+)
 from .state import state
 
 if TYPE_CHECKING:
@@ -44,8 +47,8 @@ if TYPE_CHECKING:
     from jinja2 import Template
 
     from ..models.comm_manager import CommManager
-    from ..widgets.base import Widget
     from ..viewable import Viewable
+    from ..widgets.base import Widget
     from .location import Location
 
 #---------------------------------------------------------------------
@@ -69,8 +72,10 @@ def push(doc: 'Document', comm: 'Comm', binary: bool = True) -> None:
     comm.send(msg.header_json)
     comm.send(msg.metadata_json)
     comm.send(msg.content_json)
-    for header, payload in msg.buffers:
-        comm.send(json.dumps(header))
+    for buffer in msg.buffers:
+        header = json.dumps(buffer.ref)
+        payload = buffer.to_bytes()
+        comm.send(header)
         comm.send(buffers=[payload])
 
 def push_on_root(ref: str):
@@ -84,7 +89,10 @@ DOC_NB_JS: Template = _env.get_template("doc_nb_js.js")
 AUTOLOAD_NB_JS: Template = _env.get_template("autoload_panel_js.js")
 NB_TEMPLATE_BASE: Template = _env.get_template('nb_template.html')
 
-def _autoload_js(bundle, configs, requirements, exports, skip_imports, ipywidget, load_timeout=5000):
+def _autoload_js(
+    *, bundle, configs, requirements, exports, skip_imports, ipywidget,
+    reloading=False, load_timeout=5000
+):
     config = {'packages': {}, 'paths': {}, 'shim': {}}
     for conf in configs:
         for key, c in conf.items():
@@ -92,12 +100,14 @@ def _autoload_js(bundle, configs, requirements, exports, skip_imports, ipywidget
     return AUTOLOAD_NB_JS.render(
         bundle    = bundle,
         force     = True,
+        reloading = reloading,
         timeout   = load_timeout,
         config    = config,
         requirements = requirements,
         exports   = exports,
         skip_imports = skip_imports,
-        ipywidget = ipywidget
+        ipywidget = ipywidget,
+        version = bokeh.__version__
     )
 
 def html_for_render_items(docs_json, render_items, template=None, template_variables={}):
@@ -132,7 +142,7 @@ def render_template(
     document: 'Document', comm: Optional['Comm'] = None, manager: Optional['CommManager'] = None
 ) -> Tuple[Dict[str, str], Dict[str, Dict[str, str]]]:
     ref = document.roots[0].ref['id']
-    (docs_json, render_items) = standalone_docs_json_and_render_items(document, True)
+    (docs_json, render_items) = standalone_docs_json_and_render_items(document, suppress_callback_warning=True)
 
     # We do not want the CommManager to appear in the roots because
     # the custom template may not reference it
@@ -147,7 +157,7 @@ def render_template(
     return ({'text/html': html, EXEC_MIME: ''}, {EXEC_MIME: {'id': ref}})
 
 def render_model(
-    model: 'Model', comm: Optional['Comm'] = None
+    model: 'Model', comm: Optional['Comm'] = None, resources: str = 'cdn'
 ) -> Tuple[Dict[str, str], Dict[str, Dict[str, str]]]:
     if not isinstance(model, Model):
         raise ValueError("notebook_content expects a single Model instance")
@@ -155,7 +165,13 @@ def render_model(
 
     target = model.ref['id']
 
-    (docs_json, [render_item]) = standalone_docs_json_and_render_items([model], True)
+    if not state._is_pyodide and resources == 'server':
+        # ALERT: Replace with better approach before Bokeh 3.x compatible release
+        dist_url = '/panel-preview/static/extensions/panel/'
+        patch_model_css(model, dist_url=dist_url)
+        model.document._template_variables['dist_url'] = dist_url
+
+    (docs_json, [render_item]) = standalone_docs_json_and_render_items([model], suppress_callback_warning=True)
     div = div_for_render_item(render_item)
     render_json = render_item.to_json()
     requirements = [pnext._globals[ext] for ext in pnext._loaded_extensions
@@ -177,14 +193,16 @@ def render_model(
 
 
 def render_mimebundle(
-    model: 'Model', doc: 'Document', comm: 'Comm', manager: Optional['CommManager'] = None,
-    location: Optional['Location'] = None
+    model: 'Model', doc: 'Document', comm: 'Comm',
+    manager: Optional['CommManager'] = None,
+    location: Optional['Location'] = None,
+    resources: str = 'cdn'
 ) -> Tuple[Dict[str, str], Dict[str, Dict[str, str]]]:
     """
     Displays bokeh output inside a notebook using the PyViz display
     and comms machinery.
     """
-    if not isinstance(model, LayoutDOM):
+    if not isinstance(model, Model):
         raise ValueError('Can only render bokeh LayoutDOM models')
     add_to_doc(model, doc, True)
     if manager is not None:
@@ -192,7 +210,7 @@ def render_mimebundle(
     if location is not None:
         loc = location._get_model(doc, model, model, comm)
         doc.add_root(loc)
-    return render_model(model, comm)
+    return render_model(model, comm, resources)
 
 
 def mimebundle_to_html(bundle: Dict[str, Any]) -> str:
@@ -209,6 +227,90 @@ def mimebundle_to_html(bundle: Dict[str, Any]) -> str:
         html += '\n<script type="application/javascript">{js}</script>'.format(js=js)
     return html
 
+
+def require_components():
+    """
+    Returns JS snippet to load the required dependencies in the classic
+    notebook using REQUIRE JS.
+    """
+    from ..config import config
+
+    configs, requirements, exports = [], [], {}
+    js_requires = []
+
+    for qual_name, model in Model.model_class_reverse_map.items():
+        # We need to enable Models from Panel as well as Panel extensions
+        # like awesome_panel_extensions.
+        # The Bokeh models do not have "." in the qual_name
+        if "." in qual_name:
+            js_requires.append(model)
+
+    from ..reactive import ReactiveHTML
+    js_requires += list(param.concrete_descendents(ReactiveHTML).values())
+
+    for export, js in config.js_files.items():
+        name = js.split('/')[-1].replace('.min', '').split('.')[-2]
+        conf = {'paths': {name: js[:-3]}, 'exports': {name: export}}
+        js_requires.append(conf)
+
+    skip_import = {}
+    for model in js_requires:
+        if hasattr(model, '__js_skip__'):
+            skip_import.update(model.__js_skip__)
+
+        if not (hasattr(model, '__js_require__') or isinstance(model, dict)):
+            continue
+
+        if isinstance(model, dict):
+            model_require = model
+        else:
+            model_require = dict(model.__js_require__)
+
+        model_exports = model_require.pop('exports', {})
+        if not any(model_require == config for config in configs):
+            configs.append(model_require)
+
+        for req in list(model_require.get('paths', [])):
+            if isinstance(req, tuple):
+                model_require['paths'] = dict(model_require['paths'])
+                model_require['paths'][req[0]] = model_require['paths'].pop(req)
+
+            reqs = req[1] if isinstance(req, tuple) else (req,)
+            for r in reqs:
+                if r not in requirements:
+                    requirements.append(r)
+                    if r in model_exports:
+                        exports[r] = model_exports[r]
+
+    return configs, requirements, exports, skip_import
+
+
+class JupyterCommJSBinary(JupyterCommJS):
+    """
+    Extends pyviz_comms.JupyterCommJS with support for repacking
+    binary buffers.
+    """
+
+    @classmethod
+    def decode(cls, msg):
+        buffers = {i: v for i, v in enumerate(msg['buffers'])}
+        return dict(msg['content']['data'], _buffers=buffers)
+
+class JupyterCommManagerBinary(_JupyterCommManager):
+
+    client_comm = JupyterCommJSBinary
+
+
+class Mimebundle:
+    """
+    Wraps a generated mimebundle.
+    """
+    def __init__(self, mimebundle):
+        self._mimebundle = mimebundle
+
+    def _repr_mimebundle_(self, include=None, exclude=None):
+        return self._mimebundle
+
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
@@ -238,20 +340,34 @@ def block_comm() -> Iterator:
     finally:
         state._hold = False
 
-def load_notebook(inline: bool = True, load_timeout: int = 5000) -> None:
+def load_notebook(
+    inline: bool = True,
+    reloading: bool = False,
+    load_timeout: int = 5000
+) -> None:
     from IPython.display import publish_display_data
 
-    resources = INLINE if inline else CDN
+    resources = INLINE if inline and not state._is_pyodide else CDN
     prev_resources = settings.resources(default="server")
     user_resources = settings.resources._user_value is not _Unset
-    resources = Resources.from_bokeh(resources)
+    nb_endpoint = not state._is_pyodide
+    resources = Resources.from_bokeh(resources, notebook=nb_endpoint)
     try:
-        bundle = bundle_resources(None, resources)
-        bundle = Bundle.from_bokeh(bundle)
+        bundle = bundle_resources(
+            None, resources, notebook=nb_endpoint, reloading=reloading
+        )
         configs, requirements, exports, skip_imports = require_components()
         ipywidget = 'ipywidgets_bokeh' in sys.modules
-        bokeh_js = _autoload_js(bundle, configs, requirements, exports,
-                                skip_imports, ipywidget, load_timeout)
+        bokeh_js = _autoload_js(
+            bundle=bundle,
+            configs=configs,
+            requirements=requirements,
+            exports=exports,
+            skip_imports=skip_imports,
+            ipywidget=ipywidget,
+            reloading=reloading,
+            load_timeout=load_timeout
+        )
     finally:
         if user_resources:
             settings.resources = prev_resources
@@ -265,7 +381,7 @@ def load_notebook(inline: bool = True, load_timeout: int = 5000) -> None:
     bokeh.io.notebook.curstate().output_notebook()
 
     # Publish comm manager
-    CSS = (PANEL_DIR / '_templates' / 'jupyter.css').read_text()
+    CSS = (PANEL_DIR / '_templates' / 'jupyter.css').read_text(encoding='utf-8')
     JS = '\n'.join([PYVIZ_PROXY, _JupyterCommManager.js_manager, nb_mime_js])
     publish_display_data(data={LOAD_MIME: JS, 'application/javascript': JS})
     publish_display_data(data={'text/html': f'<style>{CSS}</style>'})
@@ -291,7 +407,8 @@ def show_server(panel: Any, notebook_url: str, port: int = 0) -> 'Server':
     server: bokeh.server.Server
     """
     from IPython.display import publish_display_data
-    from .server import _server_url, _origin_url, get_server
+
+    from .server import _origin_url, _server_url, get_server
 
     if callable(notebook_url):
         origin = notebook_url(None)
@@ -318,7 +435,7 @@ def show_server(panel: Any, notebook_url: str, port: int = 0) -> 'Server':
     })
     return server
 
-def show_embed(
+def render_embed(
     panel, max_states: int = 1000, max_opts: int = 3, json: bool = False,
     json_prefix: str = '', save_path: str = './', load_path: Optional[str] = None,
     progress: bool = True, states: Dict[Widget, List[Any]] = {}
@@ -348,7 +465,6 @@ def show_embed(
     states: dict (default={})
       A dictionary specifying the widget values to embed for each widget
     """
-    from IPython.display import publish_display_data
     from ..config import config
 
     doc = Document()
@@ -358,9 +474,13 @@ def show_embed(
         embed_state(panel, model, doc, max_states, max_opts,
                     json, json_prefix, save_path, load_path, progress,
                     states)
-    publish_display_data(*render_model(model))
+    return Mimebundle(render_model(model))
 
-def ipywidget(obj: Any, **kwargs: Any):
+def show_embed(panel, *args, **kwargs):
+    from IPython.display import publish_display_data
+    return publish_display_data(render_embed(panel, *args, **kwargs))
+
+def ipywidget(obj: Any, doc=None, **kwargs: Any):
     """
     Returns an ipywidget model which renders the Panel object.
 
@@ -370,6 +490,8 @@ def ipywidget(obj: Any, **kwargs: Any):
     ---------
     obj: object
       Any Panel object or object which can be rendered with Panel
+    doc: bokeh.Document
+        Bokeh document the bokeh model will be attached to.
     **kwargs: dict
       Keyword arguments passed to the pn.panel utility function
 
@@ -378,8 +500,10 @@ def ipywidget(obj: Any, **kwargs: Any):
     Returns an ipywidget model which renders the Panel object.
     """
     from jupyter_bokeh.widgets import BokehModel
+
     from ..pane import panel
-    model = panel(obj, **kwargs).get_root()
+    doc = doc if doc else Document()
+    model = panel(obj, **kwargs).get_root(doc=doc)
     widget = BokehModel(model, combine_events=True)
     if hasattr(widget, '_view_count'):
         widget._view_count = 0
@@ -398,7 +522,7 @@ def ipywidget(obj: Any, **kwargs: Any):
                         obj._cleanup(current[0])
                     except Exception:
                         pass
-                new_model = obj.get_root()
+                new_model = obj.get_root(doc=widget._document)
                 widget.update_from_model(new_model)
                 current[:] = [new_model]
         widget.observe(view_count_changed, '_view_count')
