@@ -32,7 +32,6 @@ from bokeh.document import Document
 from bokeh.document.locking import UnlockedDocumentProxy
 from bokeh.io import curdoc as _curdoc
 from pyviz_comms import CommManager as _CommManager
-from typing_extensions import Literal
 
 from ..util import base64url_decode, parse_timedelta
 from .logging import LOG_SESSION_RENDERED, LOG_USER_MSG
@@ -42,16 +41,19 @@ _state_logger = logging.getLogger('panel.state')
 if TYPE_CHECKING:
     from concurrent.futures import Future
 
+    from bokeh.document.models import ImportedStyleSheet
     from bokeh.model import Model
     from bokeh.server.contexts import BokehSessionContext
     from bokeh.server.server import Server
     from IPython.display import DisplayHandle
     from pyviz_comms import Comm
     from tornado.ioloop import IOLoop
+    from typing_extensions import Literal
 
     from ..template.base import BaseTemplate
     from ..viewable import Viewable
     from ..widgets.indicators import BooleanIndicator
+    from .browser import BrowserInfo
     from .callbacks import PeriodicCallback
     from .location import Location
     from .notifications import NotificationArea
@@ -142,6 +144,10 @@ class _state(param.Parameterized):
     _kernels = {}
     _ipykernels: ClassVar[WeakKeyDictionary[Document, Any]] = WeakKeyDictionary()
 
+    # Locations
+    _browser: ClassVar[BrowserInfo | None] = None # Global BrowserInfo, e.g. for notebook context
+    _browsers: ClassVar[WeakKeyDictionary[Document, BrowserInfo]] = WeakKeyDictionary() # Server browser indexed by document
+
     # Locations
     _location: ClassVar[Location | None] = None # Global location, e.g. for notebook context
     _locations: ClassVar[WeakKeyDictionary[Document, Location]] = WeakKeyDictionary() # Server locations indexed by document
@@ -187,14 +193,24 @@ class _state(param.Parameterized):
 
     # Profilers
     _launching = []
-    _profiles = param.Dict(defaultdict(list))
+    _profiles = param.Dict(default=defaultdict(list))
 
     # Endpoints
     _rest_endpoints = {}
 
+    # Style cache
+    _stylesheets: ClassVar[WeakKeyDictionary[Document, Dict[str, ImportedStyleSheet]]] = WeakKeyDictionary()
+
+    # Loaded extensions
+    _extensions_: ClassVar[WeakKeyDictionary[Document, List[str]]] = WeakKeyDictionary()
+
     # Locks
     _cache_locks: ClassVar[Dict[str, threading.Lock]] = {'main': threading.Lock()}
 
+    # Sessions
+    _sessions = {}
+    _session_key_funcs = {}
+
     def __repr__(self) -> str:
         server_info = []
         for server, panel, docs in self._servers.values():
@@ -213,6 +229,13 @@ class _state(param.Parameterized):
             from tornado.ioloop import IOLoop
             return IOLoop.current()
 
+    @property
+    def _extensions(self):
+        doc = self.curdoc
+        if not (doc and doc in self._extensions_):
+            return
+        return self._extensions_[doc]
+
     @property
     def _current_thread(self) -> str | None:
         thread = threading.current_thread()
@@ -696,6 +719,24 @@ class _state(param.Parameterized):
             self._rest_endpoints[endpoint] = ([parameterized], parameters, cb)
         parameterized.param.watch(cb, parameters)
 
+    def reset(self):
+        """
+        Resets the state object killing running servers and clearing
+        any other state held by the server.
+        """
+        self.kill_all_servers()
+        self._indicators.clear()
+        self._locations.clear()
+        self._templates.clear()
+        self._views.clear()
+        self._loaded.clear()
+        self.cache.clear()
+        self._scheduled.clear()
+        if self._thread_pool is not None:
+            self._thread_pool.shutdown(wait=False)
+            self._thread_pool = None
+        self._sessions.clear()
+        self._session_key_funcs.clear()
 
     def schedule_task(
         self, name: str, callback: Callable[[], None], at: Tat =None,
@@ -851,6 +892,20 @@ class _state(param.Parameterized):
         app_url = app_url[1:] if app_url.startswith('/') else app_url
         return urljoin(self.base_url, app_url)
 
+    @property
+    def browser_info(self) -> BrowserInfo | None:
+        from ..config import config
+        from .browser import BrowserInfo
+        if config.browser_info and self.curdoc and self.curdoc.session_context and self.curdoc not in self._browsers:
+            browser = self._browsers[self.curdoc] = BrowserInfo()
+        elif self.curdoc is None:
+            if self._browser is None and config.browser_info:
+                self._browser = BrowserInfo()
+            browser = self._browser
+        else:
+            browser = self._browsers.get(self.curdoc) if self.curdoc else None
+        return browser
+
     @property
     def curdoc(self) -> Document | None:
         """
@@ -858,7 +913,8 @@ class _state(param.Parameterized):
         """
         try:
             doc = curdoc_locked()
-            if doc and doc.session_context or self._is_pyodide:
+            pyodide_session = self._is_pyodide and 'pyodide_kernel' not in sys.modules
+            if doc and (doc.session_context or pyodide_session):
                 return doc
         finally:
             curdoc = self._curdoc.get()
