@@ -1,42 +1,46 @@
 import {undisplay} from "@bokehjs/core/dom"
 import {isArray} from "@bokehjs/core/util/types"
-import {HTMLBox} from "@bokehjs/models/layouts/html_box"
 import {build_views} from "@bokehjs/core/build_views"
-import {ModelEvent, JSON} from "@bokehjs/core/bokeh_events"
+import {ModelEvent} from "@bokehjs/core/bokeh_events"
 import {div} from "@bokehjs/core/dom"
 import {Enum} from "@bokehjs/core/kinds"
 import * as p from "@bokehjs/core/properties";
 import {ColumnDataSource} from "@bokehjs/models/sources/column_data_source"
 import {TableColumn} from "@bokehjs/models/widgets/tables"
+import {Attrs} from "@bokehjs/core/types"
 
 import {debounce} from "debounce"
 
 import {comm_settings} from "./comm_manager"
 import {transform_cds_to_records} from "./data"
-import {PanelHTMLBoxView, set_size} from "./layout"
+import {HTMLBox, HTMLBoxView} from "./layout"
 
 export class TableEditEvent extends ModelEvent {
-  event_name: string = "table-edit"
-
   constructor(readonly column: string, readonly row: number, readonly pre: boolean) {
     super()
   }
 
-  protected _to_json(): JSON {
+  protected get event_values(): Attrs {
     return {model: this.origin, column: this.column, row: this.row, pre: this.pre}
   }
+
+  static {
+    this.prototype.event_name = "table-edit"
+  }
 }
 
 export class CellClickEvent extends ModelEvent {
-  event_name: string = "cell-click"
-
   constructor(readonly column: string, readonly row: number) {
     super()
   }
 
-  protected _to_json(): JSON {
+  protected get event_values(): Attrs {
     return {model: this.origin, column: this.column, row: this.row}
   }
+
+  static {
+    this.prototype.event_name = "cell-click"
+  }
 }
 
 declare const Tabulator: any;
@@ -170,7 +174,7 @@ const timestampSorter = function(a: any, b: any, _aRow: any, _bRow: any, _column
 const dateEditor = function(cell: any, onRendered: any, success: any, cancel: any) {
   //cell - the cell component for the editable cell
   //onRendered - function to call when the editor has been rendered
-  //success - function to call to pass the successfuly updated value to Tabulator
+  //success - function to call to pass the successfully updated value to Tabulator
   //cancel - function to call to abort the edit and return to a normal cell
 
   //create and style input
@@ -224,7 +228,7 @@ const dateEditor = function(cell: any, onRendered: any, success: any, cancel: an
 const datetimeEditor = function(cell: any, onRendered: any, success: any, cancel: any) {
   //cell - the cell component for the editable cell
   //onRendered - function to call when the editor has been rendered
-  //success - function to call to pass the successfuly updated value to Tabulator
+  //success - function to call to pass the successfully updated value to Tabulator
   //cancel - function to call to abort the edit and return to a normal cell
 
   //create and style input
@@ -274,25 +278,24 @@ const datetimeEditor = function(cell: any, onRendered: any, success: any, cancel
 };
 
 
-export class DataTabulatorView extends PanelHTMLBoxView {
+export class DataTabulatorView extends HTMLBoxView {
   model: DataTabulator;
   tabulator: any;
   columns: Map<string, any> = new Map();
   _tabulator_cell_updating: boolean=false
   _updating_page: boolean = true
   _updating_sort: boolean = false
-  _relayouting: boolean = false
   _selection_updating: boolean = false
   _initializing: boolean
   _lastVerticalScrollbarTopPosition: number = 0;
   _applied_styles: boolean = false
-  _building: boolean=false
+  _building: boolean = false
 
   connect_signals(): void {
     super.connect_signals()
 
     const p = this.model.properties
-    const {configuration, layout, columns, theme, groupby} = p;
+    const {configuration, layout, columns, groupby} = p;
     this.on_change([configuration, layout, groupby], debounce(() => this.invalidate_render(), 20, false))
 
     this.on_change([columns], () => {
@@ -300,8 +303,6 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       this.setHidden()
     })
 
-    this.on_change([theme], () => this.setCSS())
-
     this.connect(p.download.change, () => {
       const ftype = this.model.filename.endsWith('.json') ? "json" : "csv"
       this.tabulator.download(ftype, this.model.filename)
@@ -326,7 +327,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       }
     })
 
-    this.connect(p.styles.change, () => {
+    this.connect(p.cell_styles.change, () => {
       if (this._applied_styles)
         this.tabulator.redraw(true)
       this.setStyles()
@@ -356,6 +357,18 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.connect(this.model.source.selected.properties.indices.change, () => this.setSelection())
   }
 
+  get groupBy(): boolean | ((data: any) => string) {
+    const groupby = (data: any) => {
+      const groups = []
+      for (const g of this.model.groupby) {
+        const group = g + ': ' + data[g]
+        groups.push(group)
+      }
+      return groups.join(', ')
+    }
+    return this.model.groupby.length ? groupby : false
+  }
+
   get sorters(): any[] {
     const sorters = []
     if (this.model.sorters.length)
@@ -372,44 +385,45 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.tabulator.destroy()
     this.tabulator = null
     this.render()
-    this.relayout()
   }
 
   redraw(): void {
-    if (!this._building) {
-      if (this.tabulator.columnManager.element != null) {
-	this.tabulator.columnManager.redraw(true);
-      }
-      if (this.tabulator.rowManager.renderer != null) {
-	this.tabulator.rowManager.redraw(true);
-	this.setStyles()
-      }
+    if (this._building)
+      return
+    if (this.tabulator.columnManager.element != null) {
+      this.tabulator.columnManager.redraw(true);
+    }
+    if (this.tabulator.rowManager.renderer != null) {
+      this.tabulator.rowManager.redraw(true)
+      this.renderChildren()
+      this.setStyles()
     }
   }
 
   after_layout(): void {
     super.after_layout()
-    if (this.tabulator != null && (!this._relayouting || this._initializing))
+    if (this.tabulator != null && this._initializing)
       this.redraw()
+    this._initializing = false
   }
 
   render(): void {
     super.render()
-    const wait = this.setCSS()
-    if (wait)
-      return
     this._initializing = true
-    const container = div({class: "pnx-tabulator"})
-    set_size(container, this.model)
+    const container = div({style: "display: contents;"})
+    const el = div({class: "pnx-tabulator", style: "width: 100%; height: 100%"})
+    container.appendChild(el)
+    this.shadow_el.appendChild(container)
+
     let configuration = this.getConfiguration()
-
-    this.tabulator = new Tabulator(container, configuration)
+    this.tabulator = new Tabulator(el, configuration)
+    this.watch_stylesheets()
     this.init_callbacks()
-    this.renderChildren(true)
+  }
 
-    this.setHidden()
-
-    this.el.appendChild(container)
+  style_redraw(): void {
+    if (!this._initializing && !this._building)
+      this.redraw()
   }
 
   tableInit(): void {
@@ -429,15 +443,10 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.tabulator.on("tableBuilding", () => this.tableInit())
     this.tabulator.on("tableBuilt", () => this.tableBuilt())
 
-    // Disable frozenColumns during rendering (see https://github.com/olifolkerd/tabulator/issues/3530)
-    this.tabulator.on("dataLoading", () => {
-      this.tabulator.modules.frozenColumns.active = false
-    })
-
     // Rendering callbacks
     this.tabulator.on("selectableCheck", (row: any) => {
       const selectable = this.model.selectable_rows
-      return (selectable == null) || (selectable.indexOf(row._row.data._index) >= 0)
+      return (selectable == null) || selectable.includes(row._row.data._index)
     })
     this.tabulator.on("tooltips", (cell: any) => {
       return cell.getColumn().getField() + ": " + cell.getValue();
@@ -455,7 +464,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       this.model.filters = filters
     })
     this.tabulator.on("dataFiltered", (_: any, rows: any[]) => {
-      if (this._initializing)
+      if (this._building)
 	return
       // Ensure that after filtering empty scroll renders
       if (rows.length === 0)
@@ -482,42 +491,14 @@ export class DataTabulatorView extends PanelHTMLBoxView {
 
   tableBuilt(): void {
     this._building = false
-    this.setHidden()
     this.setSelection()
-
-    // For remote pagination initialize on tableBuilt
-    this.setMaxPage()
-    this.tabulator.setPage(this.model.page)
-    this.setStyles()
     this.renderChildren()
-    this.setGroupBy()
-    this.setFrozen()
-    this.tabulator.modules.frozenColumns.active = true
-    this.tabulator.modules.frozenColumns.layout()
-    this._initializing = false
-  }
+    this.setStyles()
 
-  relayout(): void {
-    if (this._relayouting)
-      return
-    this._relayouting = true
-    if (this.tabulator.rowManager.renderer) {
-      this.tabulator.rowManager.adjustTableSize()
+    if (this.model.pagination) {
+      this.setMaxPage()
+      this.tabulator.setPage(this.model.page)
     }
-    this.update_layout()
-    this.compute_layout()
-    if (this.root !== this) {
-      this.invalidate_layout()
-      const parent = (this as any).root._parent
-      if (parent != null && parent.relayout != null)
-        parent.relayout()
-    } else if ((this as any)._parent != null) { // HACK: Support ReactiveHTML
-      if ((this as any)._parent.relayout != null)
-        (this as any)._parent.relayout()
-      else
-        (this as any)._parent.invalidate_layout()
-    }
-    this._relayouting = false
   }
 
   requestPage(page: number, sorters: any[]): Promise<void> {
@@ -577,7 +558,11 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       pagination: this.model.pagination != null,
       paginationMode: this.model.pagination,
       paginationSize: this.model.page_size,
-      paginationInitialPage: 1
+      paginationInitialPage: 1,
+      groupBy: this.groupBy,
+      frozenRows: (row: any) => {
+	return this.model.frozen_rows.length ? this.model.frozen_rows.includes(row._row.data._index) : false
+      }
     }
     if (this.model.pagination === "remote") {
       configuration['ajaxURL'] = "http://panel.pyviz.org"
@@ -597,12 +582,12 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     }
   }
 
-  renderChildren(initializing: boolean=false): void {
+  renderChildren(): void {
     new Promise(async (resolve: any) => {
       const children = []
       for (const idx of this.model.expanded) {
-        if (idx in this.model.children)
-          children.push(this.model.children[idx])
+        if (this.model.children.has(idx))
+          children.push(this.model.children.get(idx))
       }
       await build_views(this._child_views, children, {parent: (null as any)})
       resolve(null)
@@ -611,16 +596,14 @@ export class DataTabulatorView extends PanelHTMLBoxView {
         const row = this.tabulator.getRow(r)
         this._render_row(row)
       }
-      if ((!this.model.expanded.length) && (!initializing))
-        setTimeout(() => this.relayout(), 20)
     })
   }
 
   _render_row(row: any): void {
-    const index = row._row.data._index
-    if (this.model.expanded.indexOf(index) < 0 || !(index in this.model.children))
+    const index = row._row?.data._index
+    if (!this.model.expanded.includes(index) || !this.model.children.has(index))
       return
-    const model = this.model.children[index]
+    const model = this.model.children.get(index)
     const view = this._child_views.get(model)
     if (view == null)
       return
@@ -639,8 +622,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       const neg_margin = "-" + rowEl.style.paddingLeft
       viewEl = div({style: "background-color: " + bg +"; margin-left:" + neg_margin})
     }
-    row.getElement().appendChild(viewEl);
-    view.renderTo(viewEl)
+    rowEl.appendChild(viewEl)
+    view.render_to(viewEl)
   }
 
   _expand_render(cell: any): string {
@@ -689,7 +672,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.columns = new Map()
     const config_columns: (any[] | undefined) = this.model.configuration?.columns;
     let columns = []
-    columns.push({field: '_index', frozen: true})
+    columns.push({field: '_index', frozen: true, visible: false})
     if (config_columns != null) {
       for (const column of config_columns)
         if (column.columns != null) {
@@ -764,7 +747,9 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       if (tab_column.sorter == 'timestamp') {
         tab_column.sorter = timestampSorter
       }
-
+      if (tab_column.sorter === undefined) {
+        tab_column.sorter = "string"
+      }
       const editor: any = column.editor
       const ctype = editor.type
       if (tab_column.editor != null) {
@@ -800,6 +785,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
           this.renderEditor(column, cell, onRendered, success, cancel)
         }
       }
+      tab_column.visible = (tab_column.visible != false && !this.model.hidden_columns.includes(column.field))
       tab_column.editable = () => (this.model.editable && (editor.default_view != null))
       if (tab_column.headerFilter) {
         if ((typeof tab_column.headerFilter) === 'boolean' &&
@@ -814,7 +800,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       }
       tab_column.cellClick = (_: any, cell: any) => {
         const index = cell.getData()._index
-        this.model.trigger_event(new CellClickEvent(column.field, index))
+	const event = new CellClickEvent(column.field, index)
+	this.model.trigger_event(event)
       }
       if (config_columns == null)
         columns.push(tab_column)
@@ -828,7 +815,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
         hozAlign: "center",
         cellClick: (_: any, cell: any) => {
           const index = cell.getData()._index
-          this.model.trigger_event(new CellClickEvent(col, index))
+	  const event = new CellClickEvent(col, index)
+          this.model.trigger_event(event)
         }
       }
       columns.push(button_column)
@@ -866,9 +854,9 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     const data = this.getData()
     if (this.model.pagination != null)
       this.tabulator.rowManager.setData(data, true, false)
-    else {
+    else
       this.tabulator.setData(data)
-    }
+
     this.postUpdate()
   }
 
@@ -883,11 +871,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   }
 
   postUpdate(): void {
-    if (!this.model.pagination)
-      this.setFrozen()
     this.setSelection()
-    if (this.model.height == null && this.model.pagination == null)
-      this.relayout()
   }
 
   updateOrAddData(): void {
@@ -901,8 +885,9 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   }
 
   setFrozen(): void {
-    for (const row of this.model.frozen_rows)
+    for (const row of this.model.frozen_rows) {
       this.tabulator.getRow(row).freeze()
+    }
   }
 
   updatePage(pageno: number): void {
@@ -910,25 +895,12 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       this._updating_page = true
       this.model.page = pageno
       this._updating_page = false
+      this.setStyles()
     }
   }
 
   setGroupBy(): void {
-    if (this.model.groupby.length == 0) {
-      this.tabulator.setGroupBy(false)
-      return
-    }
-    const groupby = (data: any) => {
-      const groups = []
-      for (const g of this.model.groupby) {
-        const group = g + ': ' + data[g]
-        groups.push(group)
-      }
-      return groups.join(', ')
-    }
-    // Need to call it twice, see https://github.com/olifolkerd/tabulator/issues/3666
-    this.tabulator.setGroupBy(groupby)
-    this.tabulator.setGroupBy(groupby)
+    this.tabulator.setGroupBy(this.groupBy)
   }
 
   setSorters(): void {
@@ -937,76 +909,19 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.tabulator.setSort(this.sorters)
   }
 
-  setCSS(): boolean {
-    let theme: string
-    let theme_: string
-    if (this.model.theme == "default") {
-      theme = "tabulator"
-    }
-    else {
-      if (this.model.theme == "bootstrap") {
-        theme_ = "bootstrap3"
-      }
-      else if (this.model.theme == "semantic-ui") {
-        theme_ = "semanticui"
-      }
-      else {
-        theme_ = this.model.theme
-      }
-      theme = "tabulator_" + theme_
-    }
-    const css = this.model.theme_url + theme + ".min.css"
-
-    let old_node: any = null
-    const links = document.getElementsByTagName("link")
-    const dist_index = this.model.theme_url.indexOf('dist/')
-    const start_url = this.model.theme_url.slice(0, dist_index)
-    for (const link of links) {
-      if (link.href.indexOf(start_url) >= 0) {
-        old_node = link
-        break
-      }
-    }
-
-    if (old_node != null) {
-      if (old_node.href.endsWith(css))
-        return false
-      else {
-        old_node.href = css
-        setTimeout(() => this.render(), 100)
-        return true
-      }
-    }
-    let parent_node = document.getElementsByTagName("head")[0]
-
-    const css_node: any = document.createElement('link')
-    css_node.type = 'text/css'
-    css_node.rel = 'stylesheet'
-    css_node.media = 'screen'
-    css_node.href = css
-
-    css_node.onload = () => {
-      if (!this._building) {
-	this.render()
-	this.relayout()
-      }
-    }
-    parent_node.appendChild(css_node)
-    return true
-  }
-
   setStyles(): void {
-    if (this.tabulator == null || this.tabulator.getDataCount() == 0)
+    const style_data = this.model.cell_styles.data
+    if (this.tabulator == null || this.tabulator.getDataCount() == 0 || !style_data.size)
       return
     this._applied_styles = false
-    for (const r in this.model.styles.data) {
-      const row_style = this.model.styles.data[r]
+    for (const r of style_data.keys()) {
+      const row_style = style_data.get(r)
       const row = this.tabulator.getRow(r)
       if (!row)
         continue
       const cells = row._row.cells
-      for (const c in row_style) {
-        const style = row_style[c]
+      for (const c of row_style.keys()) {
+        const style = row_style.get(c)
         const cell = cells[c]
         if (cell == null || !style.length)
           continue
@@ -1029,7 +944,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   setHidden(): void {
     for (const column of this.tabulator.getColumns()) {
       const col = column._column
-      if ((col.field == '_index') || (this.model.hidden_columns.indexOf(col.field) > -1))
+      if ((col.field == '_index') || this.model.hidden_columns.includes(col.field))
         column.hide()
       else
         column.show()
@@ -1044,10 +959,18 @@ export class DataTabulatorView extends PanelHTMLBoxView {
 
   setPage(): void {
     this.tabulator.setPage(Math.min(this.model.max_page, this.model.page))
+    if (this.model.pagination === "local") {
+      this.renderChildren()
+      this.setStyles()
+    }
   }
 
   setPageSize(): void {
     this.tabulator.setPageSize(this.model.page_size)
+    if (this.model.pagination === "local") {
+      this.renderChildren()
+      this.setStyles()
+    }
   }
 
   setSelection(): void {
@@ -1084,7 +1007,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     const selected = this.model.source.selected
     const index: number = row._row.data._index
     if (e.ctrlKey || e.metaKey) {
-      indices = this.model.source.selected.indices
+      indices = [...this.model.source.selected.indices]
     } else if (e.shiftKey && selected.indices.length) {
       const start = selected.indices[selected.indices.length-1]
       if (index>start) {
@@ -1191,9 +1114,7 @@ export namespace DataTabulator {
     selectable_rows: p.Property<number[] | null>
     source: p.Property<ColumnDataSource>
     sorters: p.Property<any[]>
-    styles: p.Property<any>
-    theme: p.Property<string>
-    theme_url: p.Property<string>
+    cell_styles: p.Property<any>
   }
 }
 
@@ -1209,7 +1130,7 @@ export class DataTabulator extends HTMLBox {
 
   static __module__ = "panel.models.tabulator"
 
-  static init_DataTabulator(): void {
+  static {
     this.prototype.default_view = DataTabulatorView;
 
     this.define<DataTabulator.Props>(({Any, Array, Boolean, Nullable, Number, Ref, String}) => ({
@@ -1237,9 +1158,7 @@ export class DataTabulator extends HTMLBox {
       selectable_rows: [ Nullable(Array(Number)), null ],
       source:         [ Ref(ColumnDataSource)       ],
       sorters:        [ Array(Any),              [] ],
-      styles:         [ Any,                     {} ],
-      theme:          [ String,            "simple" ],
-      theme_url:      [ String, "https://unpkg.com/tabulator-tables@5.3.2/dist/css/"]
+      cell_styles:    [ Any,                     {} ],
     }))
   }
 }
