@@ -5,7 +5,7 @@ bokeh model.
 from __future__ import annotations
 
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, Mapping, Optional,
+    TYPE_CHECKING, Any, ClassVar, List, Mapping, Optional,
 )
 
 import numpy as np
@@ -14,9 +14,10 @@ import param
 from bokeh.models import ColumnDataSource
 from pyviz_comms import JupyterComm
 
+from ..io.resources import CDN_DIST
 from ..util import isdatetime, lazy_load
 from ..viewable import Layoutable
-from .base import PaneBase
+from .base import ModelPane
 
 if TYPE_CHECKING:
     from bokeh.document import Document
@@ -24,7 +25,7 @@ if TYPE_CHECKING:
     from pyviz_comms import Comm
 
 
-class Plotly(PaneBase):
+class Plotly(ModelPane):
     """
     The `Plotly` pane renders Plotly plots inside a panel.
 
@@ -83,9 +84,15 @@ class Plotly(PaneBase):
 
     priority: ClassVar[float | bool | None] = 0.8
 
+    _stylesheets: ClassVar[List[str]] = [
+        f'{CDN_DIST}css/plotly.css'
+    ]
+
     _updates: ClassVar[bool] = True
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'link_figure': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'link_figure': None, 'object': None
+    }
 
     @classmethod
     def applies(cls, obj: Any) -> float | bool | None:
@@ -271,19 +278,18 @@ class Plotly(PaneBase):
         params['frames'] = json.get('frames', [])
         if layout.get('autosize') and self.sizing_mode is self.param.sizing_mode.default:
             params['sizing_mode'] = 'stretch_both'
+            if 'styles' not in params:
+                params['styles'] = {}
         return params
 
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
-        PlotlyPlot = lazy_load('panel.models.plotly', 'PlotlyPlot', isinstance(comm, JupyterComm), root)
-        model = PlotlyPlot(**self._init_params())
-        if root is None:
-            root = model
-        self._link_props(model, self._linkable_params, doc, root, comm)
-        self._models[root.ref['id']] = (model, parent)
-        return model
+        self._bokeh_model  = lazy_load(
+            'panel.models.plotly', 'PlotlyPlot', isinstance(comm, JupyterComm), root
+        )
+        return super()._get_model(doc, root, parent, comm)
 
     def _update(self, ref: str, model: Model) -> None:
         if self.object is None:
@@ -339,10 +345,14 @@ class Plotly(PaneBase):
         updates = {}
         if self.sizing_mode is self.param.sizing_mode.default and 'autosize' in layout:
             autosize = layout.get('autosize')
+            styles = dict(model.styles)
             if autosize and model.sizing_mode != 'stretch_both':
                 updates['sizing_mode'] = 'stretch_both'
+                styles['display'] = 'contents'
             elif not autosize and model.sizing_mode != 'fixed':
                 updates['sizing_mode'] = 'fixed'
+                if 'display' in styles:
+                    del styles['display']
 
         if new_sources:
             updates['data_sources'] = model.data_sources + new_sources
