@@ -10,24 +10,27 @@ import signal
 import socket
 import tempfile
 import time
+import unittest
 
 from contextlib import contextmanager
 from subprocess import PIPE, Popen
 
+import pandas as pd
 import pytest
 
 from bokeh.client import pull_session
 from bokeh.document import Document
-from bokeh.model import Model
 from pyviz_comms import Comm
 
 from panel import config, serve
 from panel.config import panel_extension
-from panel.io import state
+from panel.io.state import set_curdoc, state
 from panel.pane import HTML, Markdown
 
 CUSTOM_MARKS = ('ui', 'jupyter')
 
+config.apply_signatures = False
+
 JUPYTER_PORT = 8887
 JUPYTER_TIMEOUT = 15 # s
 JUPYTER_PROCESS = None
@@ -148,6 +151,14 @@ def document():
     return Document()
 
 
+@pytest.fixture
+def server_document():
+    doc = Document()
+    session_context = unittest.mock.Mock()
+    doc._session_context = lambda: session_context
+    with set_curdoc(doc):
+        yield doc
+
 @pytest.fixture
 def comm():
     return Comm()
@@ -161,7 +172,6 @@ def port():
 
 @pytest.fixture
 def dataframe():
-    import pandas as pd
     return pd.DataFrame({
         'int': [1, 2, 3],
         'float': [3.14, 6.28, 9.42],
@@ -299,19 +309,19 @@ def set_env_var(env_var, value):
     else:
         os.environ[env_var] = old_value
 
-
 @pytest.fixture(autouse=True)
 def module_cleanup():
     """
     Cleanup Panel extensions after each test.
     """
+    from bokeh.core.has_props import _default_resolver
     to_reset = list(panel_extension._imports.values())
-    Model.model_class_reverse_map = {
-        name: model for name, model in Model.model_class_reverse_map.items()
+
+    _default_resolver._known_models = {
+        name: model for name, model in _default_resolver._known_models.items()
         if not any(model.__module__.startswith(tr) for tr in to_reset)
     }
 
-
 @pytest.fixture(autouse=True)
 def server_cleanup():
     """
@@ -320,17 +330,7 @@ def server_cleanup():
     try:
         yield
     finally:
-        state.kill_all_servers()
-        state._indicators.clear()
-        state._locations.clear()
-        state._templates.clear()
-        state._views.clear()
-        state._loaded.clear()
-        state.cache.clear()
-        state._scheduled.clear()
-        if state._thread_pool is not None:
-            state._thread_pool.shutdown(wait=False)
-            state._thread_pool = None
+        state.reset()
 
 @pytest.fixture(autouse=True)
 def cache_cleanup():
@@ -360,6 +360,17 @@ def threads():
     finally:
         config.nthreads = None
 
+@pytest.fixture
+def reuse_sessions():
+    config.reuse_sessions = True
+    try:
+        yield
+    finally:
+        config.reuse_sessions = False
+        config.session_key_func = None
+        state._sessions.clear()
+        state._session_key_funcs.clear()
+
 @pytest.fixture
 def nothreads():
     yield
