@@ -4,12 +4,12 @@ import pytest
 from panel.io import block_comm
 from panel.layout import Row
 from panel.links import CallbackGenerator
+from panel.tests.util import check_layoutable_properties
 from panel.widgets import (
-    CompositeWidget, Dial, FileDownload, FloatSlider, LinearGauge,
-    TextInput, Terminal, ToggleGroup, Tqdm, Widget
+    CompositeWidget, Dial, FileDownload, FloatSlider, LinearGauge, Terminal,
+    TextInput, ToggleGroup, Tqdm, Widget,
 )
 from panel.widgets.tables import BaseTable
-from panel.tests.util import check_layoutable_properties
 
 excluded = (
     BaseTable, CompositeWidget, Dial, FileDownload, LinearGauge,
@@ -27,16 +27,28 @@ def test_widget_signature(widget):
     parameters = signature(widget).parameters
     assert len(parameters) == 1
 
+@pytest.mark.parametrize('widget', all_widgets)
+def test_widget_untracked_watchers(widget, document, comm):
+    # Ensures internal code correctly detects
+    try:
+        widg = widget()
+    except ImportError:
+        pytest.skip("Dependent library could not be imported.")
+    watchers = [
+        w for pwatchers in widg._param_watchers.values()
+        for awatchers in pwatchers.values() for w in awatchers
+    ]
+    assert len([wfn for wfn in watchers if wfn not in widg._internal_callbacks and not hasattr(wfn.fn, '_watcher_name')]) == 0
 
 @pytest.mark.parametrize('widget', all_widgets)
-def test_widget_linkable_params(widget):
+def test_widget_linkable_params(widget, document, comm):
     w = widget()
     controls = w.controls(jslink=True)
     layout = Row(w, controls)
 
     try:
         CallbackGenerator.error = True
-        layout.get_root()
+        layout.get_root(document, comm)
     finally:
         CallbackGenerator.error = False
 
