@@ -8,12 +8,15 @@ from __future__ import annotations
 import math
 
 from typing import (
-    TYPE_CHECKING, Any, Callable, ClassVar, List, Mapping, Optional, Tuple,
-    Type,
+    TYPE_CHECKING, Any, Callable, ClassVar, Dict, List, Mapping, Optional,
+    Tuple, Type,
 )
 
 import param  # type: ignore
 
+from bokeh.models import ImportedStyleSheet
+
+from .._param import Margin
 from ..layout.base import Row
 from ..reactive import Reactive
 from ..viewable import Layoutable, Viewable
@@ -43,7 +46,7 @@ class Widget(Reactive):
 
     width = param.Integer(default=None, bounds=(0, None))
 
-    margin = param.Parameter(default=(5, 10), doc="""
+    margin = Margin(default=(5, 10), doc="""
         Allows to create additional space around the component. May
         be specified as a two-tuple of the form (vertical, horizontal)
         or a four-tuple (top, right, bottom, left).""")
@@ -93,24 +96,25 @@ class Widget(Reactive):
         )
         return layout[0]
 
+    def _process_param_change(self, params: Dict[str, Any]) -> Dict[str, Any]:
+        params = super()._process_param_change(params)
+        if self._widget_type is not None and 'stylesheets' in params:
+            css = getattr(self._widget_type, '__css__', [])
+            params['stylesheets'] = [
+                ImportedStyleSheet(url=ss) for ss in css
+            ] + params['stylesheets']
+        return params
+
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
-        model = self._widget_type(**self._process_param_change(self._init_params()))
-        if root is None:
-            root = model
-        # Link parameters and bokeh model
-        values = self.param.values()
-        properties = self._filter_properties(list(self._process_param_change(values)))
+        model = self._widget_type(**self._get_properties(doc))
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
-        self._link_props(model, properties, doc, root, comm)
+        self._link_props(model, self._linked_properties, doc, root, comm)
         return model
 
-    def _filter_properties(self, properties: List[str]) -> List[str]:
-        ignored = list(Layoutable.param)+['loading']
-        return [p for p in properties if p not in ignored]
-
     def _get_embed_state(
         self, root: 'Model', values: Optional[List[Any]] = None, max_opts: int = 3
     ) -> Tuple['Widget', 'Model', List[Any], Callable[['Model'], Any], str, str]:
@@ -152,6 +156,8 @@ class CompositeWidget(Widget):
 
     _composite_type: ClassVar[Type[ListPanel]] = Row
 
+    _linked_properties: ClassVar[Tuple[str]] = ()
+
     __abstract = True
 
     def __init__(self, **params):
@@ -161,9 +167,15 @@ class CompositeWidget(Widget):
                   if getattr(self, p) is not None}
         if layout.get('width', self.width) is None and 'sizing_mode' not in layout:
             layout['sizing_mode'] = 'stretch_width'
+        if layout.get('sizing_mode') not in (None, 'fixed') and layout.get('width'):
+            min_width = layout.pop('width')
+            if not layout.get('min_width'):
+                layout['min_width'] = min_width
         self._composite = self._composite_type(**layout)
         self._models = self._composite._models
-        self.param.watch(self._update_layout_params, layout_params)
+        self._internal_callbacks.append(
+            self.param.watch(self._update_layout_params, layout_params)
+        )
 
     def _update_layout_params(self, *events: param.parameterized.Event) -> None:
         updates = {event.name: event.new for event in events}
@@ -200,8 +212,7 @@ class CompositeWidget(Widget):
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
         model = self._composite._get_model(doc, root, parent, comm)
-        if root is None:
-            root = parent = model
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
         return model
 
