@@ -25,6 +25,7 @@ from bokeh.models.widgets import (
 
 from ..config import config
 from ..io import state
+from ..io.resources import CDN_DIST
 from ..layout import Column, Panel, Row
 from ..util import (
     datetime_as_utctimestamp, edit_readonly, param_reprs, value_as_date,
@@ -355,7 +356,7 @@ class DiscreteSlider(CompositeWidget, _SliderBase):
                              'is one of the declared options.'
                              % self.value)
 
-        self._text = StaticText(margin=(5, 0, 0, 5), style={'white-space': 'nowrap'})
+        self._text = StaticText(margin=(5, 0, 0, 5), styles={'white-space': 'nowrap'})
         self._slider = None
         self._composite = Column(self._text, self._slider)
         self._update_options()
@@ -610,7 +611,7 @@ class IntRangeSlider(RangeSlider):
         The lower bound.""")
 
     end = param.Integer(default=1, doc="""
-        The uppper bound.""")
+        The upper bound.""")
 
     step = param.Integer(default=1, doc="""
         The step size""")
@@ -797,7 +798,8 @@ class _EditableContinuousSlider(CompositeWidget):
         self._slider.param.watch(self._sync_value, 'value_throttled')
 
         self._value_edit = self._input_widget(
-            margin=0, align='end', css_classes=['slider-edit']
+            margin=0, align='end', css_classes=['slider-edit'],
+            stylesheets=[f'{CDN_DIST}css/editable_slider.css']
         )
         self._value_edit.param.watch(self._sync_value, 'value')
         self._value_edit.param.watch(self._sync_value, 'value_throttled')
@@ -1016,16 +1018,20 @@ class EditableRangeSlider(CompositeWidget, _SliderBase):
         self._slider = RangeSlider(margin=(0, 0, 5, 0), show_value=False)
         self._slider.param.watch(self._sync_value, 'value')
         self._slider.param.watch(self._sync_value, 'value_throttled')
-        self._start_edit = FloatInput(min_width=50, margin=0, format=self.format,
-                                      css_classes=['slider-edit'])
-        self._end_edit = FloatInput(min_width=50, margin=(0, 0, 0, 10), format=self.format,
-                                    css_classes=['slider-edit'])
+        self._start_edit = FloatInput(
+            css_classes=['slider-edit'], stylesheets=[f'{CDN_DIST}css/editable_slider.css'],
+            min_width=50, margin=0, format=self.format
+        )
+        self._end_edit = FloatInput(
+            css_classes=['slider-edit'], stylesheets=[f'{CDN_DIST}css/editable_slider.css'],
+            min_width=50, margin=(0, 0, 0, 10), format=self.format
+        )
         self._start_edit.param.watch(self._sync_start_value, 'value')
         self._start_edit.param.watch(self._sync_start_value, 'value_throttled')
         self._end_edit.param.watch(self._sync_end_value, 'value')
         self._end_edit.param.watch(self._sync_end_value, 'value_throttled')
 
-        sep = StaticText(value='...', margin=(0, 2, 0, 2), align='end')
+        sep = StaticText(value='...', margin=(0, 5, 0, 5), align='end')
         edit = Row(self._label, self._start_edit, sep, self._end_edit,
                    sizing_mode='stretch_width', margin=0)
         self._composite.extend([edit, self._slider])
