@@ -18,12 +18,21 @@ offline. Whether this is secure and confidential enough for your use
 case is up to you to evaluate.
 """
 
+from __future__ import annotations
+
+from typing import (
+    TYPE_CHECKING, ClassVar, Mapping, Type,
+)
+
 import param
 
 from ..models.speech_to_text import SpeechToText as _BkSpeechToText
 from .base import Widget
 from .button import BUTTON_TYPES
 
+if TYPE_CHECKING:
+    from bokeh.model import Model
+
 BUTTON_TYPES = BUTTON_TYPES+['light', 'dark']
 
 
@@ -384,14 +393,12 @@ class SpeechToText(Widget):
         List used to transfer the serialized grammars from server to
         browser.""")
 
-    _widget_type = _BkSpeechToText
-
-    _rename = {
-        "value": None,
-        "grammars": None,
-        "_grammars": "grammars",
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'grammars': None, '_grammars': 'grammars', 'name': None, 'value': None,
     }
 
+    _widget_type: ClassVar[Type[Model]] = _BkSpeechToText
+
     def __init__(self, **params):
         super().__init__(**params)
         if self.grammars:
@@ -402,7 +409,7 @@ class SpeechToText(Widget):
         # multiple actions
         return f"SpeechToText(name='{self.name}')"
 
-    @param.depends("grammars", watch=True)
+    @param.depends('grammars', watch=True)
     def _update_grammars(self):
         with param.edit_constant(self):
             if self.grammars:
@@ -410,14 +417,14 @@ class SpeechToText(Widget):
             else:
                 self._grammars = []
 
-    @param.depends("results", watch=True)
+    @param.depends('results', watch=True)
     def _update_results(self):
         # pylint: disable=unsubscriptable-object
         with param.edit_constant(self):
-            if self.results and "alternatives" in self.results[-1]:
-                self.value = (self.results[-1]["alternatives"][0]["transcript"]).lstrip()
+            if self.results and 'alternatives' in self.results[-1]:
+                self.value = (self.results[-1]['alternatives'][0]['transcript']).lstrip()
             else:
-                self.value = ""
+                self.value = ''
 
     @property
     def results_deserialized(self):
@@ -434,16 +441,16 @@ class SpeechToText(Widget):
         Convenience method for ease of use
         """
         if not self.results:
-            return "No results"
-        html = "<div class='pn-speech-recognition-result'>"
+            return 'No results'
+        html = '<div class="pn-speech-recognition-result">'
         total = len(self.results) - 1
         for index, result in enumerate(reversed(self.results_deserialized)):
             if len(self.results) > 1:
-                html += f"<h3>Result {total-index}</h3>"
-            html += f"<span>Is Final: {result.is_final}</span><br/>"
+                html += f'<h3>Result {total-index}</h3>'
+            html += f'<span>Is Final: {result.is_final}</span><br/>'
             for index2, alternative in enumerate(result.alternatives):
                 if len(result.alternatives) > 1:
-                    html += f"<h4>Alternative {index2}</h4>"
+                    html += f'<h4>Alternative {index2}</h4>'
                 html += f"""
                 <span>Confidence: {alternative.confidence:.2f}</span>
                 </br>
@@ -451,5 +458,5 @@ class SpeechToText(Widget):
                   <strong>{alternative.transcript}</strong>
                 </p>
                 """
-        html += "</div>"
+        html += '</div>'
         return html
